import requests
import json

from .auth import AuthClient


class ConverterClient:

    def __init__(self, converter_url):
        self.converter_url = converter_url

    def start(self, job_config):
        """
        :param job_config: json object of job configuration
        :return: job_id
        """
        keycloak_token = AuthClient.fetch_personal_token()
        converter_response = requests.post(
            f'{self.converter_url}/jobs',
            headers={'Authorization': 'Bearer %s' % keycloak_token, 'Content-type': 'application/json'},
            data=json.dumps(job_config)
        )
        converter_response.raise_for_status()
        return converter_response

    def get_job_summary(self, job_id):
        """
        :param job_id: job id returned after starting converter job using start()
        :return: job summary
        """
        keycloak_token = AuthClient.fetch_personal_token()

        job_summary = requests.get(
            f'{self.converter_url}/jobs/{job_id}/execution-summary',
            headers={'Authorization': 'Bearer %s' % keycloak_token, 'Content-type': 'application/json'}
        )
        job_summary.raise_for_status()
        return job_summary

    def stop_job(self, job_id):
        """
        :param job_id:
        :return:
        """
        keycloak_token = AuthClient.fetch_personal_token()

        job_status = requests.post(
            f'{self.converter_url}/jobs/{job_id}/stop',
            headers={'Authorization': 'Bearer %s' % keycloak_token, 'Content-type': 'application/json'}
        )
        job_status.raise_for_status()
        return job_status



from jupyterhub.services.auth import HubAuth
from google.oauth2.credentials import Credentials
import os
import requests
from functools import partial


class AuthClient:
    """
    A client class that connects to the AuthHandler to retrieve user and auth state info
    """
    @staticmethod
    def fetch_local_user():
        # Helps getting the correct ssl configs
        hub = HubAuth()
        response = requests.get(os.environ['LOCAL_USER_PATH'],
                                headers={
                                    'Authorization': 'token %s' % hub.api_token
                                }, cert=(hub.certfile, hub.keyfile), verify=hub.client_ca, allow_redirects=False)
        if response.status_code == 200:
            return response.json()
        else:
            raise AuthError

    @staticmethod
    def fetch_personal_token():
        try:
            return AuthClient.fetch_local_user()['access_token']
        except AuthError as err:
            err.print_warning()

    @staticmethod
    def fetch_google_token():
        try:
            return AuthClient.fetch_local_user()['exchanged_tokens']['google']['access_token']
        except AuthError as err:
            err.print_warning()

    @staticmethod
    def fetch_google_credentials():
        credentials = Credentials(
            token=AuthClient.fetch_google_token(),
            token_uri="https://oauth2.googleapis.com/token",
        )

        def _refresh(self, request):
            self.token = AuthClient.fetch_google_token()

        credentials.refresh = partial(_refresh, credentials)
        return credentials

    @staticmethod
    def is_ready():
        return 'LOCAL_USER_PATH' in os.environ


class AuthError(Exception):
    """This exception class is used when the communication with the custom auth handler fails.
    This is normally due to stale auth session."""

    def print_warning(self):
        from IPython.core.display import display, HTML
        display(HTML('Your session has timed out. Please <a href="/hub/login">log in</a> to continue.'))




import json

import requests

from .auth import AuthClient


class CollectorClient:

    def __init__(self, collector_url):
        self.collector_url = collector_url

    def start(self, specification):
        keycloak_token = AuthClient.fetch_personal_token()
        collector_response = requests.put(
            self.collector_url,
            headers={'Authorization': 'Bearer %s' % keycloak_token, 'Content-type': 'application/json'},
            data=json.dumps(specification)
        )
        collector_response.raise_for_status()
        print("Task initiated successfully! Check running tasks with the running_tasks() method.")
        return collector_response

    def running_tasks(self):
        keycloak_token = AuthClient.fetch_personal_token()
        collector_response = requests.get(self.collector_url, headers={'Authorization': 'Bearer %s' % keycloak_token})
        collector_response.raise_for_status()
        return collector_response

    def stop(self, task_id):
        keycloak_token = AuthClient.fetch_personal_token()
        collector_response = requests.delete(f'{self.collector_url}/{task_id}',
                                             headers={'Authorization': 'Bearer %s' % keycloak_token})
        if collector_response.status_code == 400:
            print("Collector task with id: " + task_id + " cannot be stopped! Maybe it was already stopped or completed")
        else:
            print("Collector task with id: " + task_id + " stopped successfully")

        return collector_response



import gcsfs


class GCSFileSystem(gcsfs.GCSFileSystem):

    def __init__(self, token=None, **kwargs):
        super().__init__(token=token, **kwargs)
        # Temporary bug fix for https://issues.apache.org/jira/browse/ARROW-7867
        # Spark writes an empty file to GCS (to mimic a folder structure) before writing partitioned data
        # Resolve this by ignoring the "empty" file when reading partitioned parquet files
        from pyarrow.parquet import EXCLUDED_PARQUET_PATHS
        from pyarrow.parquet import ParquetManifest
        EXCLUDED_PARQUET_PATHS.add('')
        ParquetManifest._should_silently_exclude = GCSFileSystem._should_silently_exclude

    def isdir(self, path):
        info = super(gcsfs.GCSFileSystem, self).info(path)
        return info["type"] == "directory"

    @staticmethod
    def _should_silently_exclude(self, file_name):
        from pyarrow.parquet import EXCLUDED_PARQUET_PATHS
        return (file_name.endswith('.crc') or  # Checksums
                file_name.endswith('_$folder$') or  # HDFS directories in S3
                file_name.startswith('.') or  # Hidden files starting with .
                file_name.startswith('_') or  # Hidden files starting with _
                '.tmp' in file_name or  # Temp files
                file_name in EXCLUDED_PARQUET_PATHS)




import json
import requests

from .auth import AuthClient


class GuardianClient:
    @staticmethod
    def call_api(api_endpoint_url, maskinporten_client_id, scopes, guardian_endpoint_url="https://guardian.dapla.ssb.no/maskinporten/access-token", keycloak_token=""):
        """
        Call an external API using the maskinporten guardian
        :param api_endpoint_url: URL to the target API
        :param maskinporten_client_id: the Maskinporten client id
        :param scopes: the Maskinporten scopes
        :param guardian_endpoint_url: URL to the Maskinporten Guardian
        :param keycloak_token: the user's personal Keycloak token. Automatic fetch attempt will be made if left empty.
        :return: the endpoint json response
        """
        if keycloak_token == "":
            keycloak_token = AuthClient.fetch_personal_token()
        body = {
            "maskinportenClientId": maskinporten_client_id,
            "scopes": scopes
        }
        body = json.dumps(body)
        maskinporten_token = GuardianClient.get_guardian_token(guardian_endpoint_url, keycloak_token, body)
        api_response = requests.get(api_endpoint_url,
                                    headers={
                                        'Authorization': 'Bearer %s' % maskinporten_token,
                                        'Accept': 'application/json'
                                    })
        if api_response.status_code == 200:
            return api_response.json()
        else:
            raise RuntimeError(f'Error calling target API ({api_endpoint_url}): Status code <{api_response.status_code} - {api_response.text or api_response.reason}>')

    @staticmethod
    def get_guardian_token(guardian_endpoint, keycloak_token, body):
        """
        Retrieve access token from makinporten guardian
        :param guardian_endpoint: URL to the maskinporten guardian
        :param keycloak_token: the user's Keycloak token
        :param body: maskinporten request body
        :return: the maskinporten access token
        """
        guardian_response = requests.post(guardian_endpoint,
                                          headers={
                                              'Authorization': 'Bearer %s' % keycloak_token,
                                              'Content-type': 'application/json'
                                          }, data=body)
        if guardian_response.status_code == 200:
            return guardian_response.json()['accessToken']
        else:
            raise RuntimeError(f'Error getting guardian token: <{guardian_response.status_code}: {guardian_response.text or guardian_response.reason}>')




import io
import os
import re

from setuptools import setup
from setuptools import find_packages


def read(filename):
    filename = os.path.join(os.path.dirname(__file__), filename)
    text_type = type(u"")
    with io.open(filename, mode="r", encoding='utf-8') as fd:
        return re.sub(text_type(r':[a-z]+:`~?(.*?)`'), text_type(r'``\1``'), fd.read())


DEPENDENCIES = ["pandas", "jupyterhub", "gcsfs", "ipython", "lxml", "pyarrow"]

# This call to setup() does all the work
setup(
    name="dapla-toolbelt",
    version="1.3.0",
    license='Apache Software License',
    description="Python module for use within Jupyterlab notebooks, specifically aimed for Statistics Norway's data "
                "platform called Dapla",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    url="https://github.com/statisticsnorway/dapla-toolbelt",
    author="Statistics Norway",
    author_email="old@ssb.no",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9"
    ],
    packages=find_packages(exclude=('tests', 'examples')),
    include_package_data=True,
    install_requires=DEPENDENCIES
)
